@@ -19,7 +19,6 @@ libcloud provides a unified interface to the cloud computing resources.
 :var __version__: Current version of libcloud
 """
 
-import logging
 import os
 import codecs
 import atexit
@@ -28,14 +27,10 @@ from libcloud.base import DriverType  # NOQA
 from libcloud.base import DriverTypeFactoryMap  # NOQA
 from libcloud.base import get_driver  # NOQA
 
-
-try:
-    import paramiko  # NOQA
-    have_paramiko = True
-except ImportError:
-    have_paramiko = False
-
 try:
+    # TODO: This import is slow and adds overhead in situations when no
+    # requests are made but it's necessary for detecting bad version of
+    # requests
     import requests  # NOQA
     have_requests = True
 except ImportError:
@@ -46,7 +41,7 @@ __all__ = [
     'enable_debug'
 ]
 
-__version__ = '3.2.0'
+__version__ = '3.4.1'
 
 
 def enable_debug(fo):
@@ -82,6 +77,7 @@ def _init_once():
     This also checks for known environment/dependency incompatibilities.
     """
     path = os.getenv('LIBCLOUD_DEBUG')
+
     if path:
         mode = 'a'
 
@@ -97,8 +93,24 @@ def _init_once():
         fo = codecs.open(path, mode, encoding='utf8')
         enable_debug(fo)
 
+        # NOTE: We use lazy import to avoid unnecessary import time overhead
+        try:
+            import paramiko  # NOQA
+            have_paramiko = True
+        except ImportError:
+            have_paramiko = False
+
         if have_paramiko and hasattr(paramiko.util, 'log_to_file'):
-            paramiko.util.log_to_file(filename=path, level=logging.DEBUG)
+            import logging
+
+            # paramiko always tries to open file path in append mode which
+            # won't work with /dev/{stdout, stderr} so we just ignore those
+            # errors
+            try:
+                paramiko.util.log_to_file(filename=path, level=logging.DEBUG)
+            except OSError as e:
+                if "illegal seek" not in str(e).lower():
+                    raise e
 
     # check for broken `yum install python-requests`
     if have_requests and requests.__version__ == '2.6.0':
