@@ -14,6 +14,7 @@
 # limitations under the License.
 
 from typing import Dict
+from typing import Optional
 from typing import Type
 
 import base64
@@ -339,6 +340,8 @@ class AWSRequestSignerAlgorithmV4(AWSRequestSigner):
                           for k, v in sorted(headers.items())]) + '\n'
 
     def _get_payload_hash(self, method, data=None):
+        if data is UnsignedPayloadSentinel:
+            return UNSIGNED_PAYLOAD
         if method in ('POST', 'PUT'):
             if data:
                 if hasattr(data, 'next') or hasattr(data, '__next__'):
@@ -367,8 +370,12 @@ class AWSRequestSignerAlgorithmV4(AWSRequestSigner):
         ])
 
 
+class UnsignedPayloadSentinel:
+    pass
+
+
 class SignedAWSConnection(AWSTokenConnection):
-    version = None
+    version = None  # type: Optional[str]
 
     def __init__(self, user_id, key, secure=True, host=None, port=None,
                  url=None, timeout=None, proxy_url=None, token=None,
