@@ -33,7 +33,8 @@ from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlparse
 from libcloud.utils.py3 import urlencode
 
-from libcloud.utils.misc import lowercase_keys, retry
+from libcloud.utils.misc import lowercase_keys
+from libcloud.utils.retry import Retry
 from libcloud.common.exceptions import exception_from_message
 from libcloud.common.types import LibcloudError, MalformedResponseError
 from libcloud.http import LibcloudConnection, HttpLibResponseProxy
@@ -59,6 +60,13 @@ __all__ = [
 # Module level variable indicates if the failed HTTP requests should be retried
 RETRY_FAILED_HTTP_REQUESTS = False
 
+# Set to True to allow double slashes in the URL path. This way
+# morph_action_hook() won't strip potentially double slashes in the URLs.
+# This is to support scenarios such as this one -
+# https://github.com/apache/libcloud/issues/1529.
+# We default it to False for backward compatibility reasons.
+ALLOW_PATH_DOUBLE_SLASHES = False
+
 
 class LazyObject(object):
     """An object that doesn't get initialized until accessed."""
@@ -307,6 +315,7 @@ class Connection(object):
 
     responseCls = Response
     rawResponseCls = RawResponse
+    retryCls = Retry
     connection = None
     host = '127.0.0.1'  # type: str
     port = 443
@@ -493,7 +502,8 @@ class Connection(object):
         self.ua.append(token)
 
     def request(self, action, params=None, data=None, headers=None,
-                method='GET', raw=False, stream=False, json=None):
+                method='GET', raw=False, stream=False, json=None,
+                retry_failed=None):
         """
         Request a given `action`.
 
@@ -528,6 +538,10 @@ class Connection(object):
                     and allow streaming of the response data
                     (for downloading large files)
 
+        :param retry_failed: True if failed requests should be retried. This
+                              argument can override module level constant and
+                              environment variable value on per-request basis.
+
         :return: An :class:`Response` instance.
         :rtype: :class:`Response` instance
 
@@ -545,6 +559,11 @@ class Connection(object):
         retry_enabled = os.environ.get('LIBCLOUD_RETRY_FAILED_HTTP_REQUESTS',
                                        False) or RETRY_FAILED_HTTP_REQUESTS
 
+        # Method level argument has precedence over module level constant and
+        # environment variable
+        if retry_failed is not None:
+            retry_enabled = retry_failed
+
         action = self.morph_action_hook(action)
         self.action = action
         self.method = method
@@ -603,9 +622,9 @@ class Connection(object):
                     stream=stream)
             else:
                 if retry_enabled:
-                    retry_request = retry(timeout=self.timeout,
-                                          retry_delay=self.retry_delay,
-                                          backoff=self.backoff)
+                    retry_request = self.retryCls(retry_delay=self.retry_delay,
+                                                  timeout=self.timeout,
+                                                  backoff=self.backoff)
                     retry_request(self.connection.request)(method=method,
                                                            url=url,
                                                            body=data,
@@ -620,7 +639,7 @@ class Connection(object):
 
             if errno == -5:
                 # Throw a more-friendly exception on "no address associated
-                # with hostname" error. This error could simpli indicate that
+                # with hostname" error. This error could simply indicate that
                 # "host" Connection class attribute is set to an incorrect
                 # value
                 class_name = self.__class__.__name__
@@ -653,8 +672,23 @@ class Connection(object):
         return response
 
     def morph_action_hook(self, action):
+        """
+        Here we strip any duplicated leading or traling slashes to
+        prevent typos and other issues where some APIs don't correctly
+        handle double slashes.
+
+        Keep in mind that in some situations, "/" is a vallid path name
+        so we have a module flag which disables this behavior
+        (https://github.com/apache/libcloud/issues/1529).
+        """
+        if ALLOW_PATH_DOUBLE_SLASHES:
+            # Special case to support scenarios where double slashes are
+            # valid - e.g. for S3 paths - /bucket//path1/path2.txt
+            return self.request_path + action
+
         url = urlparse.urljoin(self.request_path.lstrip('/').rstrip('/') +
                                '/', action.lstrip('/'))
+
         if not url.startswith('/'):
             return '/' + url
         else:
@@ -962,19 +996,6 @@ class BaseDriver(object):
         self.key = key
         self.secret = secret
         self.secure = secure
-        args = [self.key]
-
-        if self.secret is not None:
-            args.append(self.secret)
-
-        args.append(secure)
-
-        if host is not None:
-            args.append(host)
-
-        if port is not None:
-            args.append(port)
-
         self.api_version = api_version
         self.region = region
 
@@ -983,8 +1004,23 @@ class BaseDriver(object):
                             'retry_delay': kwargs.pop('retry_delay', None),
                             'backoff': kwargs.pop('backoff', None),
                             'proxy_url': kwargs.pop('proxy_url', None)})
-        self.connection = self.connectionCls(*args, **conn_kwargs)
 
+        args = [self.key]
+
+        if self.secret is not None:
+            args.append(self.secret)
+
+        args.append(secure)
+
+        host = conn_kwargs.pop('host', None) or host
+
+        if host is not None:
+            args.append(host)
+
+        if port is not None:
+            args.append(port)
+
+        self.connection = self.connectionCls(*args, **conn_kwargs)
         self.connection.driver = self
         self.connection.connect()
 
