@@ -493,10 +493,10 @@ class DNSDriver(BaseDriver):
         records = zone.list_records()
         records = sorted(records, key=Record._get_numeric_id)
 
-        date = datetime.datetime.now().strftime('%Y-%m-%d %H:%m:%S')
+        date = datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')
         values = {'version': __version__, 'date': date}
 
-        lines.append('; Generated by Libcloud v%(version)s on %(date)s' %
+        lines.append('; Generated by Libcloud v%(version)s on %(date)s UTC' %
                      values)
         lines.append('$ORIGIN %(domain)s.' % {'domain': zone.domain})
         lines.append('$TTL %(domain_ttl)s\n' % {'domain_ttl': zone.ttl})
