@@ -20,7 +20,8 @@ __all__ = [
 import itertools
 import json
 
-from libcloud.common.base import JsonResponse, ConnectionUserAndKey
+from libcloud.common.base import ConnectionKey, ConnectionUserAndKey
+from libcloud.common.base import JsonResponse
 from libcloud.common.types import InvalidCredsError, LibcloudError
 from libcloud.dns.base import DNSDriver, Zone, Record
 from libcloud.dns.types import Provider, RecordType
@@ -96,6 +97,10 @@ class CloudFlareDNSResponse(JsonResponse):
         1061: (ZoneAlreadyExistsError, ['zone_id']),
         1002: (RecordDoesNotExistError, ['record_id']),
         81053: (RecordAlreadyExistsError, ['record_id']),
+        # 81057: The record already exists.
+        81057: (RecordAlreadyExistsError, []),
+        # 81058: A record with those settings already exists.
+        81058: (RecordAlreadyExistsError, ['record_id']),
     }
 
     def success(self):
@@ -132,16 +137,26 @@ class CloudFlareDNSResponse(JsonResponse):
                 'driver': self.connection.driver,
             }
 
+            if error['code'] == 81057:
+                # Record id is not available when creating a record and not
+                # updating it
+                kwargs["record_id"] = "unknown"
+
             merge_valid_keys(kwargs, context, self.connection.context)
 
             raise exception_class(**kwargs)
 
 
-class CloudFlareDNSConnection(ConnectionUserAndKey):
+class BaseDNSConnection(object):
     host = API_HOST
     secure = True
     responseCls = CloudFlareDNSResponse
 
+    def encode_data(self, data):
+        return json.dumps(data)
+
+
+class GlobalAPIKeyDNSConnection(BaseDNSConnection, ConnectionUserAndKey):
     def add_default_headers(self, headers):
         headers['Content-Type'] = 'application/json'
         headers['X-Auth-Email'] = self.user_id
@@ -149,15 +164,20 @@ class CloudFlareDNSConnection(ConnectionUserAndKey):
 
         return headers
 
-    def encode_data(self, data):
-        return json.dumps(data)
+
+class TokenDNSConnection(BaseDNSConnection, ConnectionKey):
+    def add_default_headers(self, headers):
+        headers['Content-Type'] = 'application/json'
+        headers['Authorization'] = 'Bearer %s' % self.key
+
+        return headers
 
 
 class CloudFlareDNSDriver(DNSDriver):
     type = Provider.CLOUDFLARE
     name = 'CloudFlare DNS'
     website = 'https://www.cloudflare.com'
-    connectionCls = CloudFlareDNSConnection
+    connectionCls = GlobalAPIKeyDNSConnection
 
     RECORD_TYPE_MAP = {
         RecordType.A: 'A',
@@ -175,6 +195,16 @@ class CloudFlareDNSDriver(DNSDriver):
     RECORDS_PAGE_SIZE = 100
     MEMBERSHIPS_PAGE_SIZE = 50
 
+    def __init__(self, key, secret=None, secure=True, host=None, port=None,
+                 **kwargs):
+
+        if secret is None:
+            self.connectionCls = TokenDNSConnection
+
+        super(CloudFlareDNSDriver, self).__init__(
+            key=key, secret=secret, secure=secure,
+            host=host, port=port, **kwargs)
+
     def iterate_zones(self):
         def _iterate_zones(params):
             url = '{}/zones'.format(API_BASE)
@@ -317,16 +347,24 @@ class CloudFlareDNSDriver(DNSDriver):
         or iodef.
 
         For example: 0 issue test.caa.com
+
+        NOTE: For SSHFP RecordType, data need to be in the format:
+        <algorithm> <type> <fingerprint>
         """
         url = '{}/zones/{}/dns_records'.format(API_BASE, zone.id)
 
-        data = self._normalize_record_data_for_api(type=type, data=data,)
+        content, data = self._normalize_record_data_for_api(
+            type=type, data=data
+        )
         body = {
             'type': type,
             'name': name,
-            'content': data,
+            'content': content
         }
 
+        if data:
+            body['data'] = data
+
         merge_valid_keys(body, RECORD_CREATE_ATTRIBUTES, extra)
 
         self.connection.set_context({'zone_id': zone.id})
@@ -349,14 +387,19 @@ class CloudFlareDNSDriver(DNSDriver):
         url = '{}/zones/{}/dns_records/{}'.format(API_BASE, record.zone.id,
                                                   record.id)
 
-        data = self._normalize_record_data_for_api(type=type, data=data,)
+        content, data = self._normalize_record_data_for_api(
+            type=type, data=data
+        )
         body = {
             'type': record.type if type is None else type,
             'name': record.name if name is None else name,
-            'content': record.data if data is None else data,
+            'content': content,
             'extra': record.extra or {},
         }
 
+        if data:
+            body['data'] = data
+
         merge_valid_keys(body['extra'], RECORD_UPDATE_ATTRIBUTES, extra)
 
         self.connection.set_context({'record_id': record.id})
@@ -436,18 +479,34 @@ class CloudFlareDNSDriver(DNSDriver):
 
     def _normalize_record_data_for_api(self, type, data):
         """
-        Normalize record data for "special" records such as CAA so it can be
-        used with the CloudFlare API.
+        Normalize record data for "special" records such as CAA and SSHFP
+        so it can be used with the CloudFlare API.
+
+        Keep ind mind that value for SSHFP record type onluy needs to be
+        normalized for the create / update operations.
+
+        On list operation (aka response), actual value is returned
+        normally in the "content" attribute.
         """
+        cf_data = {}
         if not data:
-            return data
+            return data, cf_data
 
         if type == RecordType.CAA:
             # Replace whitespace with \t character which CloudFlare API
             # expects
             data = data.replace(' ', '\t')
 
-        return data
+        elif type == RecordType.SSHFP:
+            _fp = data.split(" ")
+            cf_data = {
+                "algorithm": _fp[0],
+                "type": _fp[1],
+                "fingerprint": _fp[2]
+            }
+            data = None
+
+        return data, cf_data
 
     def _normalize_record_data_from_api(self, type, data):
         """
