@@ -32,7 +32,7 @@ from libcloud.compute.drivers.ec2 import EC2PlacementGroup
 from libcloud.compute.drivers.ec2 import NimbusNodeDriver, EucNodeDriver
 from libcloud.compute.drivers.ec2 import OutscaleSASNodeDriver
 from libcloud.compute.drivers.ec2 import IdempotentParamError
-from libcloud.compute.drivers.ec2 import REGION_DETAILS, VALID_EC2_REGIONS
+from libcloud.compute.drivers.ec2 import VALID_EC2_REGIONS
 from libcloud.compute.drivers.ec2 import ExEC2AvailabilityZone
 from libcloud.compute.drivers.ec2 import EC2NetworkSubnet
 from libcloud.compute.base import Node, NodeImage, NodeSize, NodeLocation
@@ -55,7 +55,7 @@ null_fingerprint = '00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:' + \
 class BaseEC2Tests(LibcloudTestCase):
 
     def test_instantiate_driver_valid_regions(self):
-        regions = REGION_DETAILS.keys()
+        regions = VALID_EC2_REGIONS
         regions = [d for d in regions if d != 'nimbus' and d != 'cn-north-1']
 
         region_endpoints = [
@@ -106,6 +106,39 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.driver = EC2NodeDriver(*EC2_PARAMS,
                                     **{'region': self.region})
 
+    def test_regions_and_signature_versions(self):
+        # Verify that correct signature versions are used for each region
+        driver = EC2NodeDriver(*EC2_PARAMS, region="us-east-1")
+        self.assertEqual(driver.signature_version, "2")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="us-east-2")
+        self.assertEqual(driver.signature_version, "4")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="eu-west-1")
+        self.assertEqual(driver.signature_version, "2")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="eu-west-3")
+        self.assertEqual(driver.signature_version, "4")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="ca-central-1")
+        self.assertEqual(driver.signature_version, "4")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="eu-central-1")
+        self.assertEqual(driver.signature_version, "4")
+
+        # Verify that signature_version can be overriden via constructor argument
+        driver = EC2NodeDriver(*EC2_PARAMS, region="us-east-1", signature_version="2")
+        self.assertEqual(driver.signature_version, "2")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="us-east-1", signature_version="4")
+        self.assertEqual(driver.signature_version, "4")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="eu-central-1", signature_version="2")
+        self.assertEqual(driver.signature_version, "2")
+
+        driver = EC2NodeDriver(*EC2_PARAMS, region="eu-central-1", signature_version="4")
+        self.assertEqual(driver.signature_version, "4")
+
     def test_instantiate_driver_with_token(self):
         token = 'temporary_credentials_token'
         driver = EC2NodeDriver(*EC2_PARAMS, **{'region': self.region, 'token': token})
@@ -896,7 +929,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_create_volume(self):
         location = self.driver.list_locations()[0]
-        vol = self.driver.create_volume(10, 'vol', location)
+        vol = self.driver.create_volume(10, 'vol', location=location)
 
         self.assertEqual(10, vol.size)
         self.assertEqual('vol', vol.name)
@@ -904,6 +937,10 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertTrue(isinstance(vol.extra['create_time'], datetime))
         self.assertEqual(False, vol.extra['encrypted'])
 
+        expected_msg = "Invalid volume type specified: invalid-type"
+        self.assertRaisesRegex(ValueError, expected_msg, self.driver.create_volume,
+                          10, 'invalid-vol', location=location, ex_volume_type='invalid-type')
+
     def test_create_encrypted_volume(self):
         location = self.driver.list_locations()[0]
         vol = self.driver.create_volume(
