@@ -16,9 +16,11 @@ from __future__ import with_statement
 
 import sys
 import tempfile
+import datetime
 
 from mock import Mock
 
+from libcloud import __version__
 from libcloud.test import unittest
 from libcloud.dns.base import DNSDriver, Zone, Record
 from libcloud.dns.types import RecordType
@@ -81,6 +83,8 @@ class BaseTestCase(unittest.TestCase):
         self.driver.list_records = Mock()
         self.driver.list_records.return_value = mock_records
 
+        now = datetime.datetime.utcnow()
+
         result = self.driver.export_zone_to_bind_format(zone=zone)
         self.driver.export_zone_to_bind_zone_file(zone=zone,
                                                   file_path=self.tmp_path)
@@ -91,6 +95,16 @@ class BaseTestCase(unittest.TestCase):
         lines1 = result.split('\n')
         lines2 = content.split('\n')
 
+        date_str = "%s-%s-%s %s:%s:%s" % (now.year, zero_pad(now.month),
+                                          zero_pad(now.day),
+                                          zero_pad(now.hour),
+                                          zero_pad(now.minute),
+                                          zero_pad(now.second))
+        expected_header = "; Generated by Libcloud v%s on %s UTC" % (__version__, date_str)
+
+        self.assertEqual(lines1[0], expected_header)
+        self.assertEqual(lines2[0], expected_header)
+
         for lines in [lines1, lines2]:
             self.assertEqual(len(lines), 2 + 1 + 9)
             assertRegex(self, lines[1], r'\$ORIGIN example\.com\.')
@@ -132,5 +146,12 @@ class BaseTestCase(unittest.TestCase):
         self.assertEqual(result, '')
 
 
+def zero_pad(value: int) -> str:
+    if value < 10:
+        return "0" + str(value)
+
+    return str(value)
+
+
 if __name__ == '__main__':
     sys.exit(unittest.main())
