@@ -21,6 +21,8 @@ from libcloud.common.types import LibcloudError
 from libcloud.test import unittest
 
 from libcloud.dns.drivers.cloudflare import CloudFlareDNSDriver
+from libcloud.dns.drivers.cloudflare import GlobalAPIKeyDNSConnection
+from libcloud.dns.drivers.cloudflare import TokenDNSConnection
 from libcloud.dns.drivers.cloudflare import ZONE_EXTRA_ATTRIBUTES
 from libcloud.dns.drivers.cloudflare import RECORD_EXTRA_ATTRIBUTES
 from libcloud.dns.types import RecordType
@@ -42,6 +44,14 @@ class CloudFlareDNSDriverTestCase(unittest.TestCase):
         CloudFlareMockHttp.use_param = 'a'
         self.driver = CloudFlareDNSDriver(*DNS_PARAMS_CLOUDFLARE)
 
+    def test_auth_key(self):
+        driver = CloudFlareDNSDriver('user@example.com', 'key')
+        self.assertEqual(driver.connectionCls, GlobalAPIKeyDNSConnection)
+
+    def test_auth_token(self):
+        driver = CloudFlareDNSDriver('sometoken')
+        self.assertEqual(driver.connectionCls, TokenDNSConnection)
+
     def test_list_record_types(self):
         record_types = self.driver.list_record_types()
         self.assertEqual(len(record_types), 9)
@@ -137,6 +147,18 @@ class CloudFlareDNSDriverTestCase(unittest.TestCase):
         self.assertEqual(record.type, 'A')
         self.assertEqual(record.data, '127.0.0.3')
 
+    def test_create_record_SSHFP_record_type(self):
+        zone = self.driver.list_zones()[0]
+
+        CloudFlareMockHttp.type = 'sshfp_record_type'
+        record = self.driver.create_record(name='test_sshfp', zone=zone,
+                                           type=RecordType.SSHFP,
+                                           data='2 1 ABCDEF12345')
+        self.assertEqual(record.id, '200')
+        self.assertEqual(record.name, 'test_sshfp')
+        self.assertEqual(record.type, 'SSHFP')
+        self.assertEqual(record.data, '2 1 ABCDEF12345')
+
     def test_create_record_CAA_record_type(self):
         zone = self.driver.list_zones()[0]
 
@@ -261,9 +283,22 @@ class CloudFlareDNSDriverTestCase(unittest.TestCase):
 
         self.assertEqual(zone, updated_zone)
 
-    def test_normalize_record_data_for_api(self):
-        result = self.driver._normalize_record_data_for_api(RecordType.CAA, '0 issue foo.bar')
-        self.assertEqual(result, '0\tissue\tfoo.bar')
+    def test_caa_normalize_record_data_for_api(self):
+        content, data = self.driver._normalize_record_data_for_api(RecordType.CAA, '0 issue foo.bar')
+        self.assertEqual(content, '0\tissue\tfoo.bar')
+        self.assertEqual(data, {})
+
+    def test_sshfp_normalize_record_data_for_api(self):
+        content, data = self.driver._normalize_record_data_for_api(RecordType.SSHFP, '2 1 ABCDEF12345')
+        self.assertIsNone(content)
+        self.assertEqual(
+            data,
+            {
+                "algorithm": "2",
+                "type": "1",
+                "fingerprint": "ABCDEF12345"
+            }
+        )
 
     def test_normalize_record_data_from_apu(self):
         result = self.driver._normalize_record_data_from_api(RecordType.CAA, '0\tissue\tfoo.bar')
@@ -340,6 +375,24 @@ class CloudFlareMockHttp(MockHttp, unittest.TestCase):
 
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
+    def _client_v4_zones_1234_dns_records_sshfp_record_type(self, method, url, body, headers):
+        if method not in ['POST']:
+            raise AssertionError('Unsupported method: %s' % (method))
+
+        url = urlparse.urlparse(url)
+        # Verify record data has been correctly normalized
+        body = json.loads(body)
+        expected_data = {
+            "algorithm": "2",
+            "type": "1",
+            "fingerprint": "ABCDEF12345"
+        }
+        self.assertEqual(body['data'], expected_data)
+
+        body = self.fixtures.load('records_{}_sshfp.json'.format(method))
+
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
     def _client_v4_zones_1234_dns_records_error_chain_error(self, method, url, body, headers):
         if method not in ['POST']:
             raise AssertionError('Unsupported method: %s' % (method))
