@@ -19,16 +19,23 @@ import socket
 import sys
 import ssl
 
+from requests.exceptions import ConnectTimeout
+
 from mock import Mock, patch
 
 import requests_mock
 
+import libcloud.common.base
+
 from libcloud.test import unittest
 from libcloud.common.base import Connection, CertificateConnection
+from libcloud.common.exceptions import RateLimitReachedError
 from libcloud.http import LibcloudBaseConnection
 from libcloud.http import LibcloudConnection
 from libcloud.http import SignedHTTPSAdapter
-from libcloud.utils.misc import retry
+from libcloud.utils.retry import Retry
+from libcloud.utils.retry import RetryForeverOnRateLimitError
+from libcloud.utils.retry import RETRY_EXCEPTIONS
 from libcloud.utils.py3 import assertRaisesRegex
 
 
@@ -40,6 +47,17 @@ class BaseConnectionClassTestCase(unittest.TestCase):
     def tearDown(self):
         if self.orig_proxy:
             os.environ['http_proxy'] = self.orig_proxy
+        elif 'http_proxy' in os.environ:
+            del os.environ['http_proxy']
+
+        libcloud.common.base.ALLOW_PATH_DOUBLE_SLASHES = False
+
+    @classmethod
+    def tearDownClass(cls):
+        if 'http_proxy' in os.environ:
+            del os.environ['http_proxy']
+
+        libcloud.common.base.ALLOW_PATH_DOUBLE_SLASHES = False
 
     def test_parse_proxy_url(self):
         conn = LibcloudBaseConnection()
@@ -167,6 +185,26 @@ class BaseConnectionClassTestCase(unittest.TestCase):
         conn = LibcloudConnection(host='localhost', port=80)
         self.assertEqual(conn.host, 'http://localhost')
 
+    def test_connection_session_timeout(self):
+        """
+        Test that the connection timeout attribute is set correctly
+        """
+        conn = LibcloudConnection(host='localhost', port=8080)
+        self.assertEqual(conn.session.timeout, 60)
+
+        conn = LibcloudConnection(host='localhost', port=8080, timeout=10)
+        self.assertEqual(conn.session.timeout, 10)
+
+    def test_connection_timeout_raised(self):
+        """
+        Test that the connection times out
+        """
+        conn = LibcloudConnection(host='localhost', port=8080, timeout=0.1)
+        # use a not-routable address to test that the connection timeouts
+        host = "http://10.255.255.1"
+        with self.assertRaises(ConnectTimeout):
+            conn.request('GET', host)
+
     def test_connection_url_merging(self):
         """
         Test that the connection class will parse URLs correctly
@@ -202,6 +240,26 @@ class BaseConnectionClassTestCase(unittest.TestCase):
         self.assertEqual(conn.morph_action_hook('/test'), '/v1/test')
         self.assertEqual(conn.morph_action_hook('test'), '/v1/test')
 
+        conn.request_path = '/a'
+        self.assertEqual(conn.morph_action_hook('//b/c.txt'), '/a/b/c.txt')
+
+        conn.request_path = '/b'
+        self.assertEqual(conn.morph_action_hook('/foo//'), '/b/foo/')
+
+        libcloud.common.base.ALLOW_PATH_DOUBLE_SLASHES = True
+
+        conn.request_path = '/'
+        self.assertEqual(conn.morph_action_hook('/'), '//')
+
+        conn.request_path = ''
+        self.assertEqual(conn.morph_action_hook('/'), '/')
+
+        conn.request_path = '/a'
+        self.assertEqual(conn.morph_action_hook('//b/c.txt'), '/a//b/c.txt')
+
+        conn.request_path = '/b'
+        self.assertEqual(conn.morph_action_hook('/foo//'), '/b/foo//')
+
     def test_connect_with_prefix(self):
         """
         Test that a connection with a base path (e.g. /v1/) will
@@ -371,53 +429,162 @@ class ConnectionClassTestCase(unittest.TestCase):
     def _raise_socket_error(self):
         raise socket.gaierror('')
 
-    def test_retry_with_sleep(self):
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_with_sleep(self, mock_connect):
+        con = Connection()
+        con.connection = Mock()
+
+        mock_connect.side_effect = socket.gaierror('')
+        retry_request = Retry(timeout=0.2, retry_delay=0.1,
+                                backoff=1)
+        self.assertRaises(socket.gaierror,
+                          retry_request(con.request), action='/')
+
+        self.assertGreater(mock_connect.call_count, 1,
+                            'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_with_timeout(self, mock_connect):
+        con = Connection()
+        con.connection = Mock()
+
+        mock_connect.side_effect = socket.gaierror('')
+        retry_request = Retry(timeout=0.2, retry_delay=0.1,
+                                backoff=1)
+        self.assertRaises(socket.gaierror,
+                          retry_request(con.request), action='/')
+
+        self.assertGreater(mock_connect.call_count, 1,
+                            'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_with_backoff(self, mock_connect):
+        con = Connection()
+        con.connection = Mock()
+
+        mock_connect.side_effect = socket.gaierror('')
+        retry_request = Retry(timeout=0.2, retry_delay=0.1,
+                                backoff=1)
+        self.assertRaises(socket.gaierror,
+                          retry_request(con.request), action='/')
+        self.assertGreater(mock_connect.call_count, 1,
+                            'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_rate_limit_error_timeout(self, mock_connect):
+        con = Connection()
+        con.connection = Mock()
+
+        mock_connect.__name__ = 'mock_connect'
+        headers = {'retry-after': 0.2}
+        mock_connect.side_effect = RateLimitReachedError(headers=headers)
+        retry_request = Retry(timeout=0.4, retry_delay=0.1,
+                                backoff=1)
+        self.assertRaises(RateLimitReachedError,
+                          retry_request(con.request), action='/')
+        self.assertEqual(mock_connect.call_count, 2,
+                        'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_rate_limit_error_forever_with_old_retry_class(self, mock_connect):
+        con = Connection()
+        con.connection = Mock()
+
+        self.retry_counter = 0
+
+        def mock_connect_side_effect(*args, **kwargs):
+            self.retry_counter += 1
+
+            if self.retry_counter < 4:
+                headers = {'retry-after': 0.1}
+                raise RateLimitReachedError(headers=headers)
+
+            return 'success'
+
+        mock_connect.__name__ = 'mock_connect'
+        headers = {'retry-after': 0.2}
+        mock_connect.side_effect = mock_connect_side_effect
+        retry_request = RetryForeverOnRateLimitError(timeout=0.1, retry_delay=0.1,
+                                backoff=1)
+        retry_request(con.request)(action='/')
+
+        # We have waited longer the timeout but continue to retry
+        result = retry_request(con.request)(action='/')
+        self.assertEqual(result, "success")
+
+        self.assertEqual(mock_connect.call_count, 5,
+                        'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_should_not_retry_on_non_defined_exception(self, mock_connect):
         con = Connection()
         con.connection = Mock()
         connect_method = 'libcloud.common.base.Connection.request'
 
-        with patch(connect_method) as mock_connect:
-            mock_connect.__name__ = 'mock_connect'
-            with self.assertRaises(socket.gaierror):
-                mock_connect.side_effect = socket.gaierror('')
-                retry_request = retry(timeout=0.2, retry_delay=0.1,
-                                      backoff=1)
-                retry_request(con.request)(action='/')
+        self.retry_counter = 0
 
-            self.assertGreater(mock_connect.call_count, 1,
-                               'Retry logic failed')
+        mock_connect.__name__ = 'mock_connect'
+        headers = {'retry-after': 0.2}
+        mock_connect.side_effect = ValueError('should not retry this '
+                                            'error')
+        retry_request = Retry(timeout=5, retry_delay=0.1, backoff=1)
 
-    def test_retry_with_timeout(self):
+        self.assertRaisesRegex(ValueError, 'should not retry this error',
+                               retry_request(con.request), action='/')
+        self.assertEqual(mock_connect.call_count, 1, 'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_rate_limit_error_success_on_second_attempt(self, mock_connect):
         con = Connection()
         con.connection = Mock()
         connect_method = 'libcloud.common.base.Connection.request'
 
-        with patch(connect_method) as mock_connect:
-            mock_connect.__name__ = 'mock_connect'
-            with self.assertRaises(socket.gaierror):
-                mock_connect.side_effect = socket.gaierror('')
-                retry_request = retry(timeout=0.2, retry_delay=0.1,
-                                      backoff=1)
-                retry_request(con.request)(action='/')
+        self.retry_counter = 0
 
-            self.assertGreater(mock_connect.call_count, 1,
-                               'Retry logic failed')
+        def mock_connect_side_effect(*args, **kwargs):
+            self.retry_counter += 1
 
-    def test_retry_with_backoff(self):
+            if self.retry_counter < 2:
+                headers = {'retry-after': 0.2}
+                raise RateLimitReachedError(headers=headers)
+
+            return 'success'
+
+        mock_connect.__name__ = 'mock_connect'
+        mock_connect.side_effect = mock_connect_side_effect
+        retry_request = Retry(timeout=0.6, retry_delay=0.1,
+                                backoff=1)
+        result = retry_request(con.request)(action='/')
+        self.assertEqual(result, "success")
+
+        self.assertEqual(mock_connect.call_count, 2,
+                        'Retry logic failed')
+
+    @patch('libcloud.common.base.Connection.request')
+    def test_retry_on_all_default_retry_exception_classes(self, mock_connect):
         con = Connection()
         con.connection = Mock()
         connect_method = 'libcloud.common.base.Connection.request'
 
-        with patch(connect_method) as mock_connect:
-            mock_connect.__name__ = 'mock_connect'
-            with self.assertRaises(socket.gaierror):
-                mock_connect.side_effect = socket.gaierror('')
-                retry_request = retry(timeout=0.2, retry_delay=0.1,
-                                      backoff=1)
-                retry_request(con.request)(action='/')
+        self.retry_counter = 0
 
-            self.assertGreater(mock_connect.call_count, 1,
-                               'Retry logic failed')
+        def mock_connect_side_effect(*args, **kwargs):
+            self.retry_counter += 1
+
+            if self.retry_counter < len(RETRY_EXCEPTIONS):
+                raise RETRY_EXCEPTIONS[self.retry_counter]
+
+            return 'success'
+
+        mock_connect.__name__ = 'mock_connect'
+        mock_connect.side_effect = mock_connect_side_effect
+        retry_request = Retry(timeout=0.6, retry_delay=0.1,
+                                backoff=1)
+        result = retry_request(con.request)(action='/')
+        self.assertEqual(result, "success")
+
+        self.assertEqual(mock_connect.call_count, len(RETRY_EXCEPTIONS),
+                        'Retry logic failed')
 
 
 class CertificateConnectionClassTestCase(unittest.TestCase):
